@@ -6,13 +6,13 @@ import (
 	"github.com/apernet/quic-go"
 )
 
-func UseBBR(conn quic.Connection) {
+func UseBBR(conn *quic.Conn) {
 	conn.SetCongestionControl(bbr.NewBbrSender(
 		bbr.DefaultClock{},
 		bbr.GetInitialPacketSize(conn.RemoteAddr()),
 	))
 }
 
-func UseBrutal(conn quic.Connection, tx uint64) {
+func UseBrutal(conn *quic.Conn, tx uint64) {
 	conn.SetCongestionControl(brutal.NewBrutalSender(tx))
 }
